﻿/**
 * SimpleParticleGenerator
 * Very simple particle emitter with gravity
 *
 * @author		Pierluigi Pesenti
 * @version		1.0
 */

/*
Licensed under the MIT License

Copyright (c) 2008 Pierluigi Pesenti

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

http://blog.oaxoa.com/
*/

package com.oaxoa.fx {

	import flash.display.Sprite;
	import flash.display.Shape;
	import flash.events.Event;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.display.Stage;
	import com.oaxoa.fx.SimpleParticle;

	public class SimpleParticleGenerator extends Sprite {

		private var _speed:Number;
		private var _angleDeg:Number;
		private var _particles:Array;
		
		public function SimpleParticleGenerator() {
			// start animating (frame based)
			addEventListener(Event.ENTER_FRAME, update);
			// start adding Particles (timer based)
			var timer:Timer=new Timer(10, 0);
			timer.addEventListener(TimerEvent.TIMER, onTimer);
			timer.start();
			_particles=new Array();
		}
		private function onTimer(event:TimerEvent):void {
			for (var i:uint=0; i<2; i++) {
				addParticle();
			}
		}
		
		public function addParticle():void {
			var shape:SimpleParticle=new SimpleParticle();
			shape.graphics.beginFill(0xffffff);
			shape.graphics.drawCircle(0,0,1);
			shape.graphics.endFill();

			addChild(shape);
			var scale:Number=2;

			var gravSpeed:Number=0;
			var acc:Number=0;
			
			var spread:Number=Math.random()*6;
			spread-=3;
			
			var angle:Number=(angleDeg+spread) / 180 * Math.PI;
			var age:uint=0;

			shape.gravSpeed=gravSpeed;
			shape.acc=acc;
			shape.speed=Math.random()*(speed*.2)+speed*.8;
			shape.angle=angle;
			shape.scale=scale;
			shape.age=age;

			shape.scaleX=shape.scaleY=scale;
			_particles.push(shape);
		}
		
		public function update(event:Event):void {
			var gravAcc:Number=0.8;
			for (var i:uint=0; i<_particles.length; i++) {
				var t=_particles[i];
				t.speed+=t.acc;
				t.gravSpeed+=gravAcc;
				var incx:Number=Math.cos(t.angle)*t.speed;
				t.x+=incx;
				t.y+=Math.sin(t.angle)*t.speed;
				t.y+=Math.sin(90)*t.gravSpeed;
				t.age++;
				t.alpha=1-(t.age/30);
				if (t.age>30 || t.y>0) {
					removeChild(t);
					_particles[i]=null;
					_particles.splice(i, 1);
				}
			}
		}
		
		// setters & getters
		
		public function set speed(n:Number):void {
			_speed=n;
		}
		public function get speed():Number {
			return _speed;
		}
		
		public function set angleDeg(n:Number):void {
			_angleDeg=n;
		}
		public function get angleDeg():Number {
			return _angleDeg;
		}
	}
}
